@@ -1,7 +1,6 @@
 from typing import Optional
 
 import requests
-from yarl import URL
 
 from markdownup.cache.cache import Cache
 from markdownup.markdownup import MarkdownUp
@@ -10,14 +9,14 @@ from markdownup.markdownup import MarkdownUp
 class BuiltinCache(Cache):
 
     def __init__(self, context: MarkdownUp):
-        self.cache_url = URL('http://' + context.config.get('cache', 'bind'))
+        self.cache_url = 'http://' + context.config.get('cache', 'bind') + '/'
 
     def get(self, key: str) -> Optional[bytes]:
-        url = str(self.cache_url / key)
+        url = self.cache_url + key
         response = requests.get(url)
         return None if response.status_code == 404 else response.text
 
     def put(self, key: str, value: bytes):
-        url = str(self.cache_url / key)
+        url = self.cache_url + key
         response = requests.put(url, value)
         response.raise_for_status()
